@@ -114,8 +114,8 @@ class Scalar(Symbol):
 class TransformerBase:
     name = "TransformerBase"
     """
-        Base class for all transformers. Implements the various operators >> + * | & 
-        as well as the compile() for rewriting complex pipelines into more simples ones.
+        Base class for all transformers. Implements the various operators ``>>`` ``+`` ``*`` ``|`` ``&`` 
+        as well as ``search()`` for executing a single query and ``compile()`` for rewriting complex pipelines into more simples ones.
     """
 
     def transform(self, topics_or_res):
@@ -126,6 +126,11 @@ class TransformerBase:
         pass
 
     def transform_iter(self, input: Iterable[dict]) -> pd.DataFrame:
+        """
+            Method that proesses an iter-dict by instantiating it as a dataframe and calling transform().
+            Returns the DataFrame returned by transform(). Used in the implementation of index() on a composed 
+            pipeline.
+        """
         return self.transform(pd.DataFrame(list(input)))
 
     def transform_gen(self, input : pd.DataFrame, batch_size=1, output_topics=False) -> Iterator[pd.DataFrame]:
@@ -287,6 +292,10 @@ class TransformerBase:
 
 class IterDictIndexerBase(TransformerBase):
     def index(self, iter : Iterable[dict], **kwargs):
+        """
+            Takes an iterable of dictionaries ("iterdict"), and consumes them. There is no return;
+            This method is typically used to implement indexers.
+        """
         pass
 
     
@@ -463,8 +472,8 @@ class CombSumTransformer(BinaryTransformerBase):
         both_cols = set(res1.columns) & set(res2.columns)
         both_cols.remove("qid")
         both_cols.remove("docno")
-        merged = res1.merge(res2, on=["qid", "docno"], suffixes=[None, "_r"])
-        merged["score"] = merged["score"] + merged["score_r"]
+        merged = res1.merge(res2, on=["qid", "docno"], suffixes=[None, "_r"], how='outer')
+        merged["score"] = merged["score"].fillna(0) + merged["score_r"].fillna(0)
         merged = merged.drop(columns=["%s_r" % col for col in both_cols])
         merged = add_ranks(merged)
         return merged
@@ -710,11 +719,6 @@ class ApplyGenericTransformer(ApplyTransformerBase):
         fn = self.fn
         return fn(inputRes)
 
-@deprecation.deprecated(deprecated_in="0.3.0",
-                        details="Please use pt.ApplyGenericTransformer")
-class LambdaPipeline(ApplyGenericTransformer):
-    pass
-
 class FeatureUnionPipeline(NAryTransformerBase):
     """
         Implements the feature union operator.
@@ -824,7 +828,11 @@ class ComposedPipeline(NAryTransformerBase):
     name = "Compose"
 
     def index(self, iter : Iterable[dict], batch_size=100):
-        from more_itertools import ichunked
+        """
+        This methods implements indexing pipelines. It is responsible for calling the transform_iter() method of its 
+        constituent transformers (except the last one) on batches of records, and the index() method on the last transformer.
+        """
+        from more_itertools import chunked
         
         if len(self.models) > 2:
             #this compose could have > 2 models. we need a composite transform() on all but the last
@@ -834,7 +842,7 @@ class ComposedPipeline(NAryTransformerBase):
         last_transformer = self.models[-1]
         
         def gen():
-            for batch in ichunked(iter, batch_size):
+            for batch in chunked(iter, batch_size):
                 batch_df = prev_transformer.transform_iter(batch)
                 for row in batch_df.itertuples(index=False):
                     yield row._asdict()
